﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Extensions.Logging;
using IceCreamShopContracts.BindingModels;
using IceCreamShopContracts.BusinessLogicsContracts;
using IceCreamShopContracts.SearchModels;
using IceCreamShopContracts.StoragesContracts;
using IceCreamShopContracts.ViewModels;
using AbstractIceCreamShopDataModels.Enums;

namespace IceCreamBusinessLogic.BusinessLogics
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;
        private readonly IOrderStorage _orderStorage;

        public OrderLogic(IOrderStorage orderStorage, ILogger<OrderLogic> logger)
        {
            _orderStorage = orderStorage;
            _logger = logger;
        }

        public bool CreateOrder(OrderBindingModel model)
        {
            CheckModel(model);
            if(model.Status != OrderStatus.Неизвестен)
            {
                _logger.LogWarning("Insert operation failed. Order status incorrect.");
                return false;
            }
            model.Status = OrderStatus.Принят;
            if (_orderStorage.Insert(model) == null)
            {
                model.Status = OrderStatus.Неизвестен;
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool DeliveryOrder(OrderBindingModel model)
        {
            return SetNewStatus(model, OrderStatus.Готов);
        }

        public bool FinishOrder(OrderBindingModel model)
        {
            return SetNewStatus(model, OrderStatus.Выдан);
        }

        public bool TakeOrderInWork(OrderBindingModel model)
        {
            return SetNewStatus(model, OrderStatus.Выполняется);
        }

        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("ReadList. OrderID:{Id}", model?.Id);
            var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        private void CheckModel(OrderBindingModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (model.Id < 0)
            {
                throw new ArgumentNullException("Некорректный идентификатор заказа", nameof(model.Id));
            }
            if (model.IceCreamId < 0)
            {
                throw new ArgumentNullException("Некорректный идентификатор мороженого", nameof(model.IceCreamId));
            }
            if (model.Count <= 0)
            {
                throw new ArgumentNullException("Количество мороженого в заказе должно быть больше 0", nameof(model.Count));
            }
            if (model.Sum <= 0)
            {
                throw new ArgumentNullException("Сумма заказа должна быть больше 0", nameof(model.Sum));
            }
            _logger.LogInformation("Order. OrderID:{Id}.Sum:{ Sum}. DocumentId: { DocumentId}", model.Id, model.Sum, model.IceCreamId);
        }

        public bool SetNewStatus(OrderBindingModel model, OrderStatus orderStatus)
        {
            // Находим статус заказа по его айди
            var vmodel = _orderStorage.GetElement(new() { Id = model.Id });
            if (vmodel == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if ((int)vmodel.Status + 1 != (int)orderStatus)
            {
                throw new InvalidOperationException($"Попытка перевести заказ не в следующий статус: " +
                    $"Текущий статус: {vmodel.Status} \n" +
                    $"Планируемый статус: {orderStatus} \n" +
                    $"Доступный статус: {(OrderStatus)((int)vmodel.Status + 1)}");
            }
            model.Status = orderStatus;
            model.DateCreate = vmodel.DateCreate;
            if (model.DateImplement == null)
                model.DateImplement = vmodel.DateImplement;
            if (vmodel.ImplementerId.HasValue)
                model.ImplementerId = vmodel.ImplementerId;
            model.IceCreamId = vmodel.IceCreamId;
            model.Sum = vmodel.Sum;
            model.Count = vmodel.Count;
            if (_orderStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
    

        public OrderViewModel? ReadElement(OrderSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id:{ Id}", model.Id);
            var element = _orderStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
    }
}
